// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj07_db.h was generated from the DDL file obj07_db.ddl.
#include "obj07_db.h"

// obj07_db_api.h was generated from the DDL file obj07_db.ddl. with 
// the -cpp option
#include "obj07_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj07 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) Several new student records are added to the database
//   3) Several new course records are added to the database
//   4) Students are registered for various courses
//   5) A list of all the students and the courses they are registered for is
//      displayed
//   6) A list of all the courses and the students thare are registered for 
//      them is displayed


// The DDL (Database Definition Language) is in obj07_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByKey
//      Db->KeyFindBy
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor->GetMembers
//      Cursor->GetOwner
//      Cursor->Connect
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj07_db' in this
// project directory.

class obj07
{
public:
    Db_obj07_db    db;

    void InsertData () throw (rdm_exception&);
    void DisplayCourseRegistrations () throw (rdm_exception&);
    void DisplayStudentRegistrations () throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
    void RegisterForCourse ( char *student_name,
                             char *course_id) throw (rdm_exception&);
};

void obj07::RegisterForCourse(
    char *student_name,
    char *course_id)
{
    Cursor_course_registration cr_cursor;
    Cursor_student             student_cursor;
    Cursor_course              course_cursor;    

    try
    {
        // Lookup the student
        student_cursor = db.Get_student_recordsBy_name();
        student_cursor.KeyFindBy_name(student_name);
        if(!student_cursor.AtRecord())
        {
            cout << "Unable to find student with the name " << student_name << endl;
            return;
        }

        // Lookup the course
        course_cursor = db.Get_course_recordsBy_id();
        course_cursor.KeyFindBy_id(course_id);
        if(!student_cursor.AtRecord())
        {
            cout << "Unable to find course with the id " << course_id << endl;
            return;
        }

        // Create the registration record
        cr_cursor = db.New_course_registration_record();

        // Association the registration record with the student and course
        cr_cursor.Connect_attended_by(course_cursor);
        cr_cursor.Connect_attends(student_cursor);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error registering a student for a course (" << e.error() << " - " << e.what() << ")" << endl;
        throw (e);
        return;
    }
    return;
}

void obj07::Initialize()
{
    // Open the database and initialize it
    try
    {
        db = db.Open("obj07_db", "x");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj07_db database" << endl;
        throw (e);
        return;
    }
    return;
}

void obj07::InsertData()
{
    struct student student_rec;    
    struct course  course_rec;

    // Create the artists, albums, and tracks in the database
    try
    {
        // Add the students
        strcpy(student_rec.name, "Jeff");
        db.New_student_recordWithFieldValues(&student_rec);

        strcpy(student_rec.name, "Brooke");
        db.New_student_recordWithFieldValues(&student_rec);

        strcpy(student_rec.name, "Jonah");
        db.New_student_recordWithFieldValues(&student_rec);

        strcpy(student_rec.name, "Norah");
        db.New_student_recordWithFieldValues(&student_rec);

        strcpy(student_rec.name, "Micah");
        db.New_student_recordWithFieldValues(&student_rec);

        // Add a few courses
        strcpy(course_rec.id, "ACCTG1A");
        strcpy(course_rec.title, "Principles of Accounting");
        db.New_course_recordWithFieldValues(&course_rec);        

        strcpy(course_rec.id, "MATH037");
        strcpy(course_rec.title, "Finite Mathematics");
        db.New_course_recordWithFieldValues(&course_rec);

        strcpy(course_rec.id, "CAOTO15");
        strcpy(course_rec.title, "Business Communications");
        db.New_course_recordWithFieldValues(&course_rec);

        strcpy(course_rec.id, "CBIS36");
        strcpy(course_rec.title, "Systems Analysis and Design");
        db.New_course_recordWithFieldValues(&course_rec);

        strcpy(course_rec.id, "IBUS1");
        strcpy(course_rec.title, "Introduction to International Business");
        db.New_course_recordWithFieldValues(&course_rec);

        // Register students for some courses
        RegisterForCourse("Jeff", "IBUS1");
        RegisterForCourse("Jeff", "CAOTO15");
        RegisterForCourse("Jeff", "ACCTG1A");
        RegisterForCourse("Brooke", "CBIS36");
        RegisterForCourse("Brooke", "IBUS1");
        RegisterForCourse("Jonah", "MATH037");
        RegisterForCourse("Jonah", "CBIS36");
        RegisterForCourse("Norah", "IBUS1");
        RegisterForCourse("Norah", "ACCTG1A");
        RegisterForCourse("Norah", "MATH037");
        RegisterForCourse("Micah", "ACCTG1A");        
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj07::DisplayCourseRegistrations()
{
    struct course              course_rec;
    struct student             student_rec;
    Cursor_student             student_cursor;
    Cursor_course              course_cursor;
    Cursor_course_registration registration_cursor;

    cout << endl << "List of students registered for each course" << endl;
    try
    {
        // Loop through each course record and display the students that are 
        // registered for it
        course_cursor = db.Get_course_recordsBy_id();
        while (!course_cursor.AfterLast())
        {
            course_cursor.GetFieldValues(&course_rec);
            cout << endl << course_rec.id << ": " << course_rec.title << endl;

            // Get the the students registered for this course
            registration_cursor = course_cursor.Get_attended_by_members();
            while (!registration_cursor.AfterLast())
            {
                student_cursor = registration_cursor.Get_attends_owner();
                student_cursor.GetFieldValues(&student_rec);
                cout << '\t' << student_rec.name << endl;
                registration_cursor++;
            }
            cout << endl;
            course_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj07::DisplayStudentRegistrations()
{
    struct course              course_rec;
    struct student             student_rec;
    Cursor_student             student_cursor;
    Cursor_course              course_cursor;
    Cursor_course_registration registration_cursor;

    cout << endl << "List of courses each student is registered for" << endl;
    try
    {
        // Loop through each student records and display the courses they are 
        // registered for it
        student_cursor = db.Get_student_recordsBy_name();
        while (!student_cursor.AfterLast())
        {
            student_cursor.GetFieldValues(&student_rec);
            cout << endl << student_rec.name << endl;

            // Get the the courses this student is registered for
            registration_cursor = student_cursor.Get_attends_members();
            while (!registration_cursor.AfterLast())
            {
                course_cursor = registration_cursor.Get_attended_by_owner();
                course_cursor.GetFieldValues(&course_rec);
                cout << '\t' << course_rec.id << ": " << course_rec.title << endl;
                registration_cursor++;
            }
            cout << endl;
            student_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj07::Cleanup()
{
    return;
}

int32_t main(void)
{
    obj07 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();
        obj.DisplayCourseRegistrations();
        obj.DisplayStudentRegistrations();
        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}
